//
//  kmpg
// 
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

//
//  A playlist manager.
//
//  This class provides the functionality needed to handle playlists, i.e.
//  lists of tracks to play. No UI specific code is in here, rather any
//  UI wishing to show/manipulate playlists can use this class to do so.
//

#ifndef PLAYLISTMGR_H
#define PLAYLISTMGR_H

#include <kurl.h>
#include <kapp.h>
#include <kconfig.h>

#include <qobject.h>
#include <qlist.h>


// A single track in the playlist
class Track 
{
   public:
      Track( KURL url ) { _url = url; };
   public:
      virtual void setPlayed( bool pl ) { _played = pl; };
      virtual bool isPlayed() { return _played; };
      virtual KURL getUrl() { return _url; };
      virtual int Length() { return _length; };
      virtual void setLength( int i ) { _length = i; };

      KURL _url; // The URL of the track
      bool _played; // Has this track been played???
      int _length; // Length of track in seconds
};

// The manager
class PlaylistMgr : public QObject
{
      Q_OBJECT
   public:
      enum PlayMode { Sequential, Random };
      enum RepeatMode { One, All, Once };

      PlaylistMgr();
      ~PlaylistMgr();
      virtual void saveProperties( KConfig* ); // Used by SM
      virtual void readProperties( KConfig* );
      void load( KURL ); // Load a playlist from a url
      void add( KURL ); // Add a playlist from a url
      void save( KURL ); // Save the playlist to a url
      void setPlayMode( PlayMode mode ) { _playmode = mode; }; // Set playmode (seq/rand)
      void setRepeatMode( RepeatMode repeat ) { _repeat = repeat; }; // Enable repeat mode
      PlayMode getPlayMode() { return _playmode; };
      RepeatMode getRepeatMode() { return _repeat; };
      int getPlayTime();
      int getTrack() { if ( isList() ) return _track + 1; else return 0; };
      int getTracks() { if ( isList() ) return _list.count(); else return 0; };
      int getPlayed();
      int getListTime();
      void setList ( bool i ) { _isList = i ;};
      bool isList() { return _isList; };
      bool areAllPlayed(); // Check if all tracks have been played
      void setAllPlayed( bool ); // Set playstatus on all tracks

   public slots:
      void clear(); // clear the playlist
      void insert( KURL ); // insert track into playlist
      void insertPlay( KURL );
      void remove( int ); // remove track from playlist
      void moveUp( int ); // move track up
      void moveDown( int ); // move track down
      void playTrack( int ); // play a certain track
      void playNext(); // play the next track
      void playPrev(); // play the previous track

   private:
      QList<Track> _list; // The list of tracks in the playlist
      PlayMode _playmode; // Which playmode, (sequential or random)
      RepeatMode _repeat; // Should the playlist repeat
      bool _changed, _isList; // Has the playlist changed since it was saved last?
      int _lastTrack, _track; // The last track played

      void doRandom(); // Play a random track

   signals:
      // These signals are emitted when specific events occur
      // within the manager.

      void sigClear(); // The playlist has been cleared
      void sigInsert( int, QString ); // Item has been inserted into playlist
      void sigUpdate( int, QString ); // Item has been updated, f.ex. name changed (ID3)
      void sigRemove( int ); // Item has been removed from playlist
      void sigMoveUp( int ); // Item has been moved up
      void sigMoveDown( int ); // Item has been moved down
      void sigSetPlayed( int, bool ); // The play status of an item has been changed
      void sigSetAllPlayed( bool );
      void sigChanged( bool ); // The changed status of the playlist has changed
      void sigTrackSelected( KURL, bool ); // A track has been selected
      void sigTrackSelected( int );
};

#endif
